package com.logicbig.example;

import javax.validation.*;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import javax.validation.executable.ExecutableValidator;
import java.lang.reflect.Constructor;
import java.util.Comparator;
import java.util.Set;

public class ConstructorParamValidationExample {

    private static class User {
        private final String name;
        private final String phone;

        public User(@NotNull String name,
                    @NotNull
                    @Pattern(regexp = "(\\d){3,3}-\\d{3,3}-\\d{4,4}",
                            message = "must match 111-111-1111 format")
                            String phone) {
            this.name = name;
            this.phone = phone;
        }

        public String getName() {
            return name;
        }

        public String getPhone() {
            return phone;
        }
    }


    public static void main(String[] args) throws NoSuchMethodException {

        Validator validator = getValidator();
        Constructor<User> constructor =
                User.class.getConstructor(String.class, String.class);
        ExecutableValidator executableValidator = validator.forExecutables();

        String userName = null;
        String userPhone = "223-223-222";

        Set<ConstraintViolation<User>> constraintViolations =
                executableValidator.validateConstructorParameters(constructor,
                        new Object[]{userName, userPhone});

        if (constraintViolations.size() > 0) {
            constraintViolations.stream().sorted(Comparator.comparing(o -> o.getPropertyPath().toString()))
                                .forEach(ConstructorParamValidationExample::printError);
        } else {
            User user = new User(userName, userPhone);
            System.out.println(user);
        }
    }

    private static Validator getValidator() {
        Configuration<?> config = Validation.byDefaultProvider().configure();
        ValidatorFactory factory = config.buildValidatorFactory();
        Validator validator = factory.getValidator();
        factory.close();
        return validator;
    }

    private static void printError(
            ConstraintViolation<User> violation) {
        System.out.println(violation.getPropertyPath() + " " + violation.getMessage());
    }
}