package com.logicbig.example;

import javax.validation.*;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import javax.validation.executable.ExecutableValidator;
import java.lang.reflect.Method;
import java.util.Comparator;
import java.util.Set;

public class MethodParamValidationExample {

    private static class Task {
        public void run (@Size(min = 3, max = 20) String name,
                         @NotNull Runnable runnable) {
            System.out.println("starting task synchronously: " + name);
            runnable.run();
        }
    }

    public static void main (String[] args) throws NoSuchMethodException {

        Validator validator = getValidator();
        Method method = Task.class.getDeclaredMethod("run",
                new Class[]{String.class, Runnable.class});
        ExecutableValidator executableValidator = validator.forExecutables();

        Task task = new Task();
        String taskName = "a";
        Runnable runnable = null;


        Set<ConstraintViolation<Task>> violations =
                            executableValidator.validateParameters(task, method,
                                                new Object[]{taskName, runnable});

        if (violations.size() > 0) {
            violations.stream().sorted(Comparator.comparing(o -> o.getPropertyPath().toString()))
                                .forEach(MethodParamValidationExample::printError);
        } else {
            task.run(taskName, runnable);
        }
    }

    private static Validator getValidator(){
        Configuration<?> config = Validation.byDefaultProvider().configure();
        ValidatorFactory factory = config.buildValidatorFactory();
        Validator validator = factory.getValidator();
        factory.close();
        return validator;
    }

    private static void printError (
                        ConstraintViolation<Task> violation) {
        System.out.println(violation.getPropertyPath() + " " + violation.getMessage());
    }
}